/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "cph-iface-mechanism.h"

#include <string.h>
#ifdef G_OS_UNIX
#  include <gio/gunixfdlist.h>
#endif

#ifdef G_ENABLE_DEBUG
#define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
#define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
#define g_marshal_value_peek_int(v)      g_value_get_int (v)
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
#define g_marshal_value_peek_long(v)     g_value_get_long (v)
#define g_marshal_value_peek_ulong(v)    g_value_get_ulong (v)
#define g_marshal_value_peek_int64(v)    g_value_get_int64 (v)
#define g_marshal_value_peek_uint64(v)   g_value_get_uint64 (v)
#define g_marshal_value_peek_enum(v)     g_value_get_enum (v)
#define g_marshal_value_peek_flags(v)    g_value_get_flags (v)
#define g_marshal_value_peek_float(v)    g_value_get_float (v)
#define g_marshal_value_peek_double(v)   g_value_get_double (v)
#define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
#define g_marshal_value_peek_param(v)    g_value_get_param (v)
#define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
#define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
#define g_marshal_value_peek_object(v)   g_value_get_object (v)
#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
#else /* !G_ENABLE_DEBUG */
/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
 *          Do not access GValues directly in your code. Instead, use the
 *          g_value_get_*() functions
 */
#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
#endif /* !G_ENABLE_DEBUG */

typedef struct
{
  GDBusArgInfo parent_struct;
  gboolean use_gvariant;
} _ExtendedGDBusArgInfo;

typedef struct
{
  GDBusMethodInfo parent_struct;
  const gchar *signal_name;
  gboolean pass_fdlist;
} _ExtendedGDBusMethodInfo;

typedef struct
{
  GDBusSignalInfo parent_struct;
  const gchar *signal_name;
} _ExtendedGDBusSignalInfo;

typedef struct
{
  GDBusPropertyInfo parent_struct;
  const gchar *hyphen_name;
  guint use_gvariant : 1;
  guint emits_changed_signal : 1;
} _ExtendedGDBusPropertyInfo;

typedef struct
{
  GDBusInterfaceInfo parent_struct;
  const gchar *hyphen_name;
} _ExtendedGDBusInterfaceInfo;

typedef struct
{
  const _ExtendedGDBusPropertyInfo *info;
  guint prop_id;
  GValue orig_value; /* the value before the change */
} ChangedProperty;

static void
_changed_property_free (ChangedProperty *data)
{
  g_value_unset (&data->orig_value);
  g_free (data);
}

static gboolean
_g_strv_equal0 (gchar **a, gchar **b)
{
  gboolean ret = FALSE;
  guint n;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  if (g_strv_length (a) != g_strv_length (b))
    goto out;
  for (n = 0; a[n] != NULL; n++)
    if (g_strcmp0 (a[n], b[n]) != 0)
      goto out;
  ret = TRUE;
out:
  return ret;
}

static gboolean
_g_variant_equal0 (GVariant *a, GVariant *b)
{
  gboolean ret = FALSE;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  ret = g_variant_equal (a, b);
out:
  return ret;
}

G_GNUC_UNUSED static gboolean
_g_value_equal (const GValue *a, const GValue *b)
{
  gboolean ret = FALSE;
  g_assert (G_VALUE_TYPE (a) == G_VALUE_TYPE (b));
  switch (G_VALUE_TYPE (a))
    {
      case G_TYPE_BOOLEAN:
        ret = (g_value_get_boolean (a) == g_value_get_boolean (b));
        break;
      case G_TYPE_UCHAR:
        ret = (g_value_get_uchar (a) == g_value_get_uchar (b));
        break;
      case G_TYPE_INT:
        ret = (g_value_get_int (a) == g_value_get_int (b));
        break;
      case G_TYPE_UINT:
        ret = (g_value_get_uint (a) == g_value_get_uint (b));
        break;
      case G_TYPE_INT64:
        ret = (g_value_get_int64 (a) == g_value_get_int64 (b));
        break;
      case G_TYPE_UINT64:
        ret = (g_value_get_uint64 (a) == g_value_get_uint64 (b));
        break;
      case G_TYPE_DOUBLE:
        {
          /* Avoid -Wfloat-equal warnings by doing a direct bit compare */
          gdouble da = g_value_get_double (a);
          gdouble db = g_value_get_double (b);
          ret = memcmp (&da, &db, sizeof (gdouble)) == 0;
        }
        break;
      case G_TYPE_STRING:
        ret = (g_strcmp0 (g_value_get_string (a), g_value_get_string (b)) == 0);
        break;
      case G_TYPE_VARIANT:
        ret = _g_variant_equal0 (g_value_get_variant (a), g_value_get_variant (b));
        break;
      default:
        if (G_VALUE_TYPE (a) == G_TYPE_STRV)
          ret = _g_strv_equal0 (g_value_get_boxed (a), g_value_get_boxed (b));
        else
          g_critical ("_g_value_equal() does not handle type %s", g_type_name (G_VALUE_TYPE (a)));
        break;
    }
  return ret;
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_resource,
        const gchar *arg_filename,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 2);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_VARIANT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectVariantFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        GVariant *arg_settings,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectVariantFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectVariantFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_variant (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_INT_BOXED_BOXED (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectIntIntBoxedBoxedFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint arg_timeout,
        gint arg_limit,
        const gchar *const *arg_include_schemes,
        const gchar *const *arg_exclude_schemes,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectIntIntBoxedBoxedFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 6);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectIntIntBoxedBoxedFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int (param_values + 2),
              g_marshal_value_peek_int (param_values + 3),
              g_marshal_value_peek_boxed (param_values + 4),
              g_marshal_value_peek_boxed (param_values + 5),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_STRING_STRING_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringStringStringStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        const gchar *arg_uri,
        const gchar *arg_ppd,
        const gchar *arg_info,
        const gchar *arg_location,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringStringStringStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 7);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringStringStringStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              g_marshal_value_peek_string (param_values + 4),
              g_marshal_value_peek_string (param_values + 5),
              g_marshal_value_peek_string (param_values + 6),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        gboolean arg_enabled,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_boolean (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringBooleanStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        gboolean arg_enabled,
        const gchar *arg_reason,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringBooleanStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 5);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringBooleanStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_boolean (param_values + 3),
              g_marshal_value_peek_string (param_values + 4),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        const gchar *arg_start,
        const gchar *arg_end,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 5);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              g_marshal_value_peek_string (param_values + 4),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOXED (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringBoxedFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        const gchar *const *arg_users,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringBoxedFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringBoxedFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_boxed (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringBoxedFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        const gchar *arg_option,
        const gchar *const *arg_values,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringBoxedFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 5);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringBoxedFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              g_marshal_value_peek_boxed (param_values + 4),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectIntFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint arg_jobid,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectIntFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectIntFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_BOOLEAN (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectIntBooleanFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint arg_jobid,
        gboolean arg_purge,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectIntBooleanFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectIntBooleanFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int (param_values + 2),
              g_marshal_value_peek_boolean (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectIntStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint arg_jobid,
        const gchar *arg_job_hold_until,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectIntStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectIntStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

/* ------------------------------------------------------------------------
 * Code for interface org.opensuse.CupsPkHelper.Mechanism
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:CphIfaceMechanism
 * @title: CphIfaceMechanism
 * @short_description: Generated C code for the org.opensuse.CupsPkHelper.Mechanism D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link> D-Bus interface in C.
 */

/* ---- Introspection data for org.opensuse.CupsPkHelper.Mechanism ---- */

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_get_IN_ARG_resource =
{
  {
    -1,
    (gchar *) "resource",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_get_IN_ARG_filename =
{
  {
    -1,
    (gchar *) "filename",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_file_get_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_file_get_IN_ARG_resource.parent_struct,
  &_cph_iface_mechanism_method_info_file_get_IN_ARG_filename.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_get_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_file_get_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_file_get_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_file_get_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_file_get_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_file_get_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_file_get =
{
  {
    -1,
    (gchar *) "FileGet",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_file_get_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_file_get_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_file_get_annotation_info_pointers
  },
  "handle-file-get",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_put_IN_ARG_resource =
{
  {
    -1,
    (gchar *) "resource",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_put_IN_ARG_filename =
{
  {
    -1,
    (gchar *) "filename",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_file_put_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_file_put_IN_ARG_resource.parent_struct,
  &_cph_iface_mechanism_method_info_file_put_IN_ARG_filename.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_file_put_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_file_put_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_file_put_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_file_put_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_file_put_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_file_put_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_file_put =
{
  {
    -1,
    (gchar *) "FilePut",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_file_put_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_file_put_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_file_put_annotation_info_pointers
  },
  "handle-file-put",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_settings =
{
  {
    -1,
    (gchar *) "settings",
    (gchar *) "a{ss}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_error.parent_struct,
  &_cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_settings.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_server_get_settings_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_server_get_settings_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_server_get_settings_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_server_get_settings =
{
  {
    -1,
    (gchar *) "ServerGetSettings",
    NULL,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_server_get_settings_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_server_get_settings_annotation_info_pointers
  },
  "handle-server-get-settings",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_server_set_settings_IN_ARG_settings =
{
  {
    -1,
    (gchar *) "settings",
    (gchar *) "a{ss}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_server_set_settings_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_server_set_settings_IN_ARG_settings.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_server_set_settings_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_server_set_settings_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_server_set_settings_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_server_set_settings_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_server_set_settings_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_server_set_settings_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_server_set_settings =
{
  {
    -1,
    (gchar *) "ServerSetSettings",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_server_set_settings_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_server_set_settings_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_server_set_settings_annotation_info_pointers
  },
  "handle-server-set-settings",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_IN_ARG_timeout =
{
  {
    -1,
    (gchar *) "timeout",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_IN_ARG_limit =
{
  {
    -1,
    (gchar *) "limit",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_IN_ARG_include_schemes =
{
  {
    -1,
    (gchar *) "include_schemes",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_IN_ARG_exclude_schemes =
{
  {
    -1,
    (gchar *) "exclude_schemes",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_devices_get_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_devices_get_IN_ARG_timeout.parent_struct,
  &_cph_iface_mechanism_method_info_devices_get_IN_ARG_limit.parent_struct,
  &_cph_iface_mechanism_method_info_devices_get_IN_ARG_include_schemes.parent_struct,
  &_cph_iface_mechanism_method_info_devices_get_IN_ARG_exclude_schemes.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_devices_get_OUT_ARG_devices =
{
  {
    -1,
    (gchar *) "devices",
    (gchar *) "a{ss}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_devices_get_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_devices_get_OUT_ARG_error.parent_struct,
  &_cph_iface_mechanism_method_info_devices_get_OUT_ARG_devices.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_devices_get_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_devices_get_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_devices_get_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_devices_get =
{
  {
    -1,
    (gchar *) "DevicesGet",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_devices_get_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_devices_get_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_devices_get_annotation_info_pointers
  },
  "handle-devices-get",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_IN_ARG_uri =
{
  {
    -1,
    (gchar *) "uri",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_IN_ARG_ppd =
{
  {
    -1,
    (gchar *) "ppd",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_IN_ARG_info =
{
  {
    -1,
    (gchar *) "info",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_IN_ARG_location =
{
  {
    -1,
    (gchar *) "location",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_IN_ARG_uri.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_IN_ARG_ppd.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_IN_ARG_info.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_IN_ARG_location.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_add_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_add_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_add_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_add =
{
  {
    -1,
    (gchar *) "PrinterAdd",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_add_annotation_info_pointers
  },
  "handle-printer-add",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_uri =
{
  {
    -1,
    (gchar *) "uri",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_ppd =
{
  {
    -1,
    (gchar *) "ppd",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_info =
{
  {
    -1,
    (gchar *) "info",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_location =
{
  {
    -1,
    (gchar *) "location",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_uri.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_ppd.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_info.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_location.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_with_ppd_file_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_add_with_ppd_file_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_add_with_ppd_file_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_add_with_ppd_file_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_add_with_ppd_file =
{
  {
    -1,
    (gchar *) "PrinterAddWithPpdFile",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_with_ppd_file_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_add_with_ppd_file_annotation_info_pointers
  },
  "handle-printer-add-with-ppd-file",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_device_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_device_IN_ARG_device =
{
  {
    -1,
    (gchar *) "device",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_device_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_device_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_device_IN_ARG_device.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_device_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_device_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_device_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_device_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_device_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_device_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_device =
{
  {
    -1,
    (gchar *) "PrinterSetDevice",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_device_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_device_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_device_annotation_info_pointers
  },
  "handle-printer-set-device",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_default_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_default_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_default_IN_ARG_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_default_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_default_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_default_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_default_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_default_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_default_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_default =
{
  {
    -1,
    (gchar *) "PrinterSetDefault",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_default_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_default_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_default_annotation_info_pointers
  },
  "handle-printer-set-default",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_enabled =
{
  {
    -1,
    (gchar *) "enabled",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_enabled.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_enabled_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_enabled_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_enabled_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_enabled_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_enabled_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_enabled_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_enabled =
{
  {
    -1,
    (gchar *) "PrinterSetEnabled",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_enabled_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_enabled_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_enabled_annotation_info_pointers
  },
  "handle-printer-set-enabled",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_enabled =
{
  {
    -1,
    (gchar *) "enabled",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_reason =
{
  {
    -1,
    (gchar *) "reason",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_enabled.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_reason.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_accept_jobs_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_accept_jobs_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_accept_jobs_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_accept_jobs_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_accept_jobs_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_accept_jobs_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_accept_jobs =
{
  {
    -1,
    (gchar *) "PrinterSetAcceptJobs",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_accept_jobs_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_accept_jobs_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_accept_jobs_annotation_info_pointers
  },
  "handle-printer-set-accept-jobs",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_rename_IN_ARG_old_name =
{
  {
    -1,
    (gchar *) "old_name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_rename_IN_ARG_new_name =
{
  {
    -1,
    (gchar *) "new_name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_rename_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_rename_IN_ARG_old_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_rename_IN_ARG_new_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_rename_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_rename_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_rename_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_rename_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_rename_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_rename_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_rename =
{
  {
    -1,
    (gchar *) "PrinterRename",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_rename_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_rename_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_rename_annotation_info_pointers
  },
  "handle-printer-rename",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_delete_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_delete_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_delete_IN_ARG_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_delete_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_delete_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_delete_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_delete_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_delete_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_delete_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_delete =
{
  {
    -1,
    (gchar *) "PrinterDelete",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_delete_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_delete_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_delete_annotation_info_pointers
  },
  "handle-printer-delete",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_add_printer_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_add_printer_IN_ARG_printer =
{
  {
    -1,
    (gchar *) "printer",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_add_printer_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_add_printer_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_class_add_printer_IN_ARG_printer.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_add_printer_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_add_printer_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_add_printer_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_class_add_printer_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_class_add_printer_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_class_add_printer_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_class_add_printer =
{
  {
    -1,
    (gchar *) "ClassAddPrinter",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_add_printer_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_add_printer_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_class_add_printer_annotation_info_pointers
  },
  "handle-class-add-printer",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_printer =
{
  {
    -1,
    (gchar *) "printer",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_printer.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_delete_printer_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_delete_printer_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_delete_printer_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_class_delete_printer_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_class_delete_printer_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_class_delete_printer_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_class_delete_printer =
{
  {
    -1,
    (gchar *) "ClassDeletePrinter",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_delete_printer_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_delete_printer_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_class_delete_printer_annotation_info_pointers
  },
  "handle-class-delete-printer",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_delete_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_delete_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_delete_IN_ARG_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_class_delete_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_class_delete_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_class_delete_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_class_delete_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_class_delete_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_class_delete_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_class_delete =
{
  {
    -1,
    (gchar *) "ClassDelete",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_delete_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_class_delete_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_class_delete_annotation_info_pointers
  },
  "handle-class-delete",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_info_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_info_IN_ARG_info =
{
  {
    -1,
    (gchar *) "info",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_info_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_info_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_info_IN_ARG_info.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_info_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_info_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_info_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_info_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_info_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_info_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_info =
{
  {
    -1,
    (gchar *) "PrinterSetInfo",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_info_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_info_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_info_annotation_info_pointers
  },
  "handle-printer-set-info",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_location_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_location_IN_ARG_location =
{
  {
    -1,
    (gchar *) "location",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_location_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_location_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_location_IN_ARG_location.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_location_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_location_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_location_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_location_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_location_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_location_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_location =
{
  {
    -1,
    (gchar *) "PrinterSetLocation",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_location_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_location_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_location_annotation_info_pointers
  },
  "handle-printer-set-location",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_shared =
{
  {
    -1,
    (gchar *) "shared",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_shared.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_shared_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_shared_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_shared_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_shared_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_shared_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_shared_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_shared =
{
  {
    -1,
    (gchar *) "PrinterSetShared",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_shared_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_shared_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_shared_annotation_info_pointers
  },
  "handle-printer-set-shared",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_start =
{
  {
    -1,
    (gchar *) "start",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_end =
{
  {
    -1,
    (gchar *) "end",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_start.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_end.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_job_sheets_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_job_sheets_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_job_sheets_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_job_sheets_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_job_sheets_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_job_sheets_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_job_sheets =
{
  {
    -1,
    (gchar *) "PrinterSetJobSheets",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_job_sheets_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_job_sheets_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_job_sheets_annotation_info_pointers
  },
  "handle-printer-set-job-sheets",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_policy =
{
  {
    -1,
    (gchar *) "policy",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_policy.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_error_policy_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_error_policy_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_error_policy_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_error_policy_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_error_policy_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_error_policy_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_error_policy =
{
  {
    -1,
    (gchar *) "PrinterSetErrorPolicy",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_error_policy_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_error_policy_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_error_policy_annotation_info_pointers
  },
  "handle-printer-set-error-policy",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_policy =
{
  {
    -1,
    (gchar *) "policy",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_policy.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_op_policy_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_op_policy_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_op_policy_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_op_policy_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_op_policy_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_op_policy_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_op_policy =
{
  {
    -1,
    (gchar *) "PrinterSetOpPolicy",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_op_policy_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_op_policy_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_op_policy_annotation_info_pointers
  },
  "handle-printer-set-op-policy",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_users =
{
  {
    -1,
    (gchar *) "users",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_users.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_allowed_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_users_allowed_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_users_allowed_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_users_allowed_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_users_allowed_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_users_allowed_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_users_allowed =
{
  {
    -1,
    (gchar *) "PrinterSetUsersAllowed",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_users_allowed_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_users_allowed_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_users_allowed_annotation_info_pointers
  },
  "handle-printer-set-users-allowed",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_users =
{
  {
    -1,
    (gchar *) "users",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_users.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_set_users_denied_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_set_users_denied_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_set_users_denied_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_set_users_denied_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_set_users_denied_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_set_users_denied_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_set_users_denied =
{
  {
    -1,
    (gchar *) "PrinterSetUsersDenied",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_users_denied_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_set_users_denied_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_set_users_denied_annotation_info_pointers
  },
  "handle-printer-set-users-denied",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_option =
{
  {
    -1,
    (gchar *) "option",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_values =
{
  {
    -1,
    (gchar *) "values",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_option.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_values.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_default_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_option_default_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_option_default_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_add_option_default_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_add_option_default_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_add_option_default_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_add_option_default =
{
  {
    -1,
    (gchar *) "PrinterAddOptionDefault",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_option_default_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_option_default_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_add_option_default_annotation_info_pointers
  },
  "handle-printer-add-option-default",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_option =
{
  {
    -1,
    (gchar *) "option",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_option.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_delete_option_default_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_delete_option_default_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_delete_option_default_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_delete_option_default_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_delete_option_default_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_delete_option_default_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_delete_option_default =
{
  {
    -1,
    (gchar *) "PrinterDeleteOptionDefault",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_delete_option_default_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_delete_option_default_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_delete_option_default_annotation_info_pointers
  },
  "handle-printer-delete-option-default",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_IN_ARG_option =
{
  {
    -1,
    (gchar *) "option",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_IN_ARG_values =
{
  {
    -1,
    (gchar *) "values",
    (gchar *) "as",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_option_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_option_IN_ARG_name.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option_IN_ARG_option.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option_IN_ARG_values.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_printer_add_option_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_printer_add_option_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_printer_add_option_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_printer_add_option_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_printer_add_option_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_printer_add_option_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_printer_add_option =
{
  {
    -1,
    (gchar *) "PrinterAddOption",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_option_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_printer_add_option_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_printer_add_option_annotation_info_pointers
  },
  "handle-printer-add-option",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_cancel_IN_ARG_jobid =
{
  {
    -1,
    (gchar *) "jobid",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_cancel_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_cancel_IN_ARG_jobid.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_cancel_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_cancel_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_cancel_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_job_cancel_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_job_cancel_annotation_info_1 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.Deprecated",
  (gchar *) "true",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_job_cancel_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_job_cancel_annotation_info_0,
  &_cph_iface_mechanism_method_job_cancel_annotation_info_1,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_job_cancel =
{
  {
    -1,
    (gchar *) "JobCancel",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_cancel_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_cancel_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_job_cancel_annotation_info_pointers
  },
  "handle-job-cancel",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_jobid =
{
  {
    -1,
    (gchar *) "jobid",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_purge =
{
  {
    -1,
    (gchar *) "purge",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_jobid.parent_struct,
  &_cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_purge.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_cancel_purge_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_cancel_purge_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_cancel_purge_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_job_cancel_purge_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_job_cancel_purge_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_job_cancel_purge_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_job_cancel_purge =
{
  {
    -1,
    (gchar *) "JobCancelPurge",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_cancel_purge_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_cancel_purge_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_job_cancel_purge_annotation_info_pointers
  },
  "handle-job-cancel-purge",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_restart_IN_ARG_jobid =
{
  {
    -1,
    (gchar *) "jobid",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_restart_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_restart_IN_ARG_jobid.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_restart_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_restart_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_restart_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_job_restart_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_job_restart_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_job_restart_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_job_restart =
{
  {
    -1,
    (gchar *) "JobRestart",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_restart_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_restart_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_job_restart_annotation_info_pointers
  },
  "handle-job-restart",
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_jobid =
{
  {
    -1,
    (gchar *) "jobid",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_job_hold_until =
{
  {
    -1,
    (gchar *) "job_hold_until",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_jobid.parent_struct,
  &_cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_job_hold_until.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _cph_iface_mechanism_method_info_job_set_hold_until_OUT_ARG_error =
{
  {
    -1,
    (gchar *) "error",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _cph_iface_mechanism_method_info_job_set_hold_until_OUT_ARG_pointers[] =
{
  &_cph_iface_mechanism_method_info_job_set_hold_until_OUT_ARG_error.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _cph_iface_mechanism_method_job_set_hold_until_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _cph_iface_mechanism_method_job_set_hold_until_annotation_info_pointers[] =
{
  &_cph_iface_mechanism_method_job_set_hold_until_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _cph_iface_mechanism_method_info_job_set_hold_until =
{
  {
    -1,
    (gchar *) "JobSetHoldUntil",
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_set_hold_until_IN_ARG_pointers,
    (GDBusArgInfo **) &_cph_iface_mechanism_method_info_job_set_hold_until_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_cph_iface_mechanism_method_job_set_hold_until_annotation_info_pointers
  },
  "handle-job-set-hold-until",
  FALSE
};

static const GDBusMethodInfo * const _cph_iface_mechanism_method_info_pointers[] =
{
  &_cph_iface_mechanism_method_info_file_get.parent_struct,
  &_cph_iface_mechanism_method_info_file_put.parent_struct,
  &_cph_iface_mechanism_method_info_server_get_settings.parent_struct,
  &_cph_iface_mechanism_method_info_server_set_settings.parent_struct,
  &_cph_iface_mechanism_method_info_devices_get.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_with_ppd_file.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_device.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_default.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_enabled.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_accept_jobs.parent_struct,
  &_cph_iface_mechanism_method_info_printer_rename.parent_struct,
  &_cph_iface_mechanism_method_info_printer_delete.parent_struct,
  &_cph_iface_mechanism_method_info_class_add_printer.parent_struct,
  &_cph_iface_mechanism_method_info_class_delete_printer.parent_struct,
  &_cph_iface_mechanism_method_info_class_delete.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_info.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_location.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_shared.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_job_sheets.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_error_policy.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_op_policy.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_users_allowed.parent_struct,
  &_cph_iface_mechanism_method_info_printer_set_users_denied.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option_default.parent_struct,
  &_cph_iface_mechanism_method_info_printer_delete_option_default.parent_struct,
  &_cph_iface_mechanism_method_info_printer_add_option.parent_struct,
  &_cph_iface_mechanism_method_info_job_cancel.parent_struct,
  &_cph_iface_mechanism_method_info_job_cancel_purge.parent_struct,
  &_cph_iface_mechanism_method_info_job_restart.parent_struct,
  &_cph_iface_mechanism_method_info_job_set_hold_until.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _cph_iface_mechanism_interface_info =
{
  {
    -1,
    (gchar *) "org.opensuse.CupsPkHelper.Mechanism",
    (GDBusMethodInfo **) &_cph_iface_mechanism_method_info_pointers,
    NULL,
    NULL,
    NULL
  },
  "mechanism",
};


/**
 * cph_iface_mechanism_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
cph_iface_mechanism_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct;
}

/**
 * cph_iface_mechanism_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #CphIfaceMechanism interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
cph_iface_mechanism_override_properties (GObjectClass *klass G_GNUC_UNUSED, guint property_id_begin)
{
  return property_id_begin - 1;
}


inline static void
cph_iface_mechanism_method_marshal_file_get (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_file_put (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_server_get_settings (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_server_set_settings (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_VARIANT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_devices_get (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_INT_BOXED_BOXED (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_add (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_STRING_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_add_with_ppd_file (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_STRING_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_device (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_default (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_enabled (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_accept_jobs (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_rename (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_delete (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_class_add_printer (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_class_delete_printer (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_class_delete (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_info (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_location (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_shared (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_job_sheets (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_error_policy (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_op_policy (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_users_allowed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOXED (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_set_users_denied (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOXED (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_add_option_default (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_delete_option_default (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_printer_add_option (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_job_cancel (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_job_cancel_purge (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_job_restart (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
cph_iface_mechanism_method_marshal_job_set_hold_until (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * CphIfaceMechanism:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>.
 */

/**
 * CphIfaceMechanismIface:
 * @parent_iface: The parent interface.
 * @handle_class_add_printer: Handler for the #CphIfaceMechanism::handle-class-add-printer signal.
 * @handle_class_delete: Handler for the #CphIfaceMechanism::handle-class-delete signal.
 * @handle_class_delete_printer: Handler for the #CphIfaceMechanism::handle-class-delete-printer signal.
 * @handle_devices_get: Handler for the #CphIfaceMechanism::handle-devices-get signal.
 * @handle_file_get: Handler for the #CphIfaceMechanism::handle-file-get signal.
 * @handle_file_put: Handler for the #CphIfaceMechanism::handle-file-put signal.
 * @handle_job_cancel: Handler for the #CphIfaceMechanism::handle-job-cancel signal.
 * @handle_job_cancel_purge: Handler for the #CphIfaceMechanism::handle-job-cancel-purge signal.
 * @handle_job_restart: Handler for the #CphIfaceMechanism::handle-job-restart signal.
 * @handle_job_set_hold_until: Handler for the #CphIfaceMechanism::handle-job-set-hold-until signal.
 * @handle_printer_add: Handler for the #CphIfaceMechanism::handle-printer-add signal.
 * @handle_printer_add_option: Handler for the #CphIfaceMechanism::handle-printer-add-option signal.
 * @handle_printer_add_option_default: Handler for the #CphIfaceMechanism::handle-printer-add-option-default signal.
 * @handle_printer_add_with_ppd_file: Handler for the #CphIfaceMechanism::handle-printer-add-with-ppd-file signal.
 * @handle_printer_delete: Handler for the #CphIfaceMechanism::handle-printer-delete signal.
 * @handle_printer_delete_option_default: Handler for the #CphIfaceMechanism::handle-printer-delete-option-default signal.
 * @handle_printer_rename: Handler for the #CphIfaceMechanism::handle-printer-rename signal.
 * @handle_printer_set_accept_jobs: Handler for the #CphIfaceMechanism::handle-printer-set-accept-jobs signal.
 * @handle_printer_set_default: Handler for the #CphIfaceMechanism::handle-printer-set-default signal.
 * @handle_printer_set_device: Handler for the #CphIfaceMechanism::handle-printer-set-device signal.
 * @handle_printer_set_enabled: Handler for the #CphIfaceMechanism::handle-printer-set-enabled signal.
 * @handle_printer_set_error_policy: Handler for the #CphIfaceMechanism::handle-printer-set-error-policy signal.
 * @handle_printer_set_info: Handler for the #CphIfaceMechanism::handle-printer-set-info signal.
 * @handle_printer_set_job_sheets: Handler for the #CphIfaceMechanism::handle-printer-set-job-sheets signal.
 * @handle_printer_set_location: Handler for the #CphIfaceMechanism::handle-printer-set-location signal.
 * @handle_printer_set_op_policy: Handler for the #CphIfaceMechanism::handle-printer-set-op-policy signal.
 * @handle_printer_set_shared: Handler for the #CphIfaceMechanism::handle-printer-set-shared signal.
 * @handle_printer_set_users_allowed: Handler for the #CphIfaceMechanism::handle-printer-set-users-allowed signal.
 * @handle_printer_set_users_denied: Handler for the #CphIfaceMechanism::handle-printer-set-users-denied signal.
 * @handle_server_get_settings: Handler for the #CphIfaceMechanism::handle-server-get-settings signal.
 * @handle_server_set_settings: Handler for the #CphIfaceMechanism::handle-server-set-settings signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>.
 */

typedef CphIfaceMechanismIface CphIfaceMechanismInterface;
G_DEFINE_INTERFACE (CphIfaceMechanism, cph_iface_mechanism, G_TYPE_OBJECT)

static void
cph_iface_mechanism_default_init (CphIfaceMechanismIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * CphIfaceMechanism::handle-file-get:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_resource: Argument passed by remote caller.
   * @arg_filename: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FileGet">FileGet()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_file_get() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-file-get",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_file_get),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_file_get,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-file-put:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_resource: Argument passed by remote caller.
   * @arg_filename: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FilePut">FilePut()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_file_put() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-file-put",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_file_put),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_file_put,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-server-get-settings:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerGetSettings">ServerGetSettings()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_server_get_settings() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-server-get-settings",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_server_get_settings),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_server_get_settings,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * CphIfaceMechanism::handle-server-set-settings:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_settings: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerSetSettings">ServerSetSettings()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_server_set_settings() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-server-set-settings",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_server_set_settings),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_server_set_settings,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_VARIANT);

  /**
   * CphIfaceMechanism::handle-devices-get:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_timeout: Argument passed by remote caller.
   * @arg_limit: Argument passed by remote caller.
   * @arg_include_schemes: Argument passed by remote caller.
   * @arg_exclude_schemes: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.DevicesGet">DevicesGet()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_devices_get() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-devices-get",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_devices_get),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_devices_get,
    G_TYPE_BOOLEAN,
    5,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRV, G_TYPE_STRV);

  /**
   * CphIfaceMechanism::handle-printer-add:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_uri: Argument passed by remote caller.
   * @arg_ppd: Argument passed by remote caller.
   * @arg_info: Argument passed by remote caller.
   * @arg_location: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAdd">PrinterAdd()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_add() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-add",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_add),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_add,
    G_TYPE_BOOLEAN,
    6,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-add-with-ppd-file:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_uri: Argument passed by remote caller.
   * @arg_ppd: Argument passed by remote caller.
   * @arg_info: Argument passed by remote caller.
   * @arg_location: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddWithPpdFile">PrinterAddWithPpdFile()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_add_with_ppd_file() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-add-with-ppd-file",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_add_with_ppd_file),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_add_with_ppd_file,
    G_TYPE_BOOLEAN,
    6,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-device:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_device: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDevice">PrinterSetDevice()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_device() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-device",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_device),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_device,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-default:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDefault">PrinterSetDefault()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_default() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-default",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_default),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_default,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-enabled:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_enabled: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetEnabled">PrinterSetEnabled()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_enabled() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-enabled",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_enabled),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_enabled,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_BOOLEAN);

  /**
   * CphIfaceMechanism::handle-printer-set-accept-jobs:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_enabled: Argument passed by remote caller.
   * @arg_reason: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetAcceptJobs">PrinterSetAcceptJobs()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_accept_jobs() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-accept-jobs",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_accept_jobs),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_accept_jobs,
    G_TYPE_BOOLEAN,
    4,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-rename:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_old_name: Argument passed by remote caller.
   * @arg_new_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterRename">PrinterRename()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_rename() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-rename",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_rename),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_rename,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-delete:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDelete">PrinterDelete()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_delete() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-delete",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_delete),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_delete,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-class-add-printer:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_printer: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassAddPrinter">ClassAddPrinter()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_class_add_printer() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-class-add-printer",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_class_add_printer),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_class_add_printer,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-class-delete-printer:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_printer: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDeletePrinter">ClassDeletePrinter()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_class_delete_printer() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-class-delete-printer",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_class_delete_printer),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_class_delete_printer,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-class-delete:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDelete">ClassDelete()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_class_delete() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-class-delete",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_class_delete),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_class_delete,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-info:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_info: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetInfo">PrinterSetInfo()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_info() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-info",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_info),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_info,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-location:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_location: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetLocation">PrinterSetLocation()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_location() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-location",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_location),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_location,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-shared:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_shared: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetShared">PrinterSetShared()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_shared() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-shared",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_shared),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_shared,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_BOOLEAN);

  /**
   * CphIfaceMechanism::handle-printer-set-job-sheets:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_start: Argument passed by remote caller.
   * @arg_end: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetJobSheets">PrinterSetJobSheets()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_job_sheets() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-job-sheets",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_job_sheets),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_job_sheets,
    G_TYPE_BOOLEAN,
    4,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-error-policy:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_policy: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetErrorPolicy">PrinterSetErrorPolicy()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_error_policy() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-error-policy",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_error_policy),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_error_policy,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-op-policy:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_policy: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetOpPolicy">PrinterSetOpPolicy()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_op_policy() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-op-policy",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_op_policy),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_op_policy,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-set-users-allowed:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_users: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersAllowed">PrinterSetUsersAllowed()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_users_allowed() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-users-allowed",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_users_allowed),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_users_allowed,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRV);

  /**
   * CphIfaceMechanism::handle-printer-set-users-denied:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_users: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersDenied">PrinterSetUsersDenied()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_set_users_denied() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-set-users-denied",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_set_users_denied),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_set_users_denied,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRV);

  /**
   * CphIfaceMechanism::handle-printer-add-option-default:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_option: Argument passed by remote caller.
   * @arg_values: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOptionDefault">PrinterAddOptionDefault()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_add_option_default() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-add-option-default",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_add_option_default),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_add_option_default,
    G_TYPE_BOOLEAN,
    4,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRV);

  /**
   * CphIfaceMechanism::handle-printer-delete-option-default:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_option: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDeleteOptionDefault">PrinterDeleteOptionDefault()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_delete_option_default() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-delete-option-default",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_delete_option_default),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_delete_option_default,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * CphIfaceMechanism::handle-printer-add-option:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_option: Argument passed by remote caller.
   * @arg_values: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOption">PrinterAddOption()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_printer_add_option() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-printer-add-option",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_printer_add_option),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_printer_add_option,
    G_TYPE_BOOLEAN,
    4,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRV);

  /**
   * CphIfaceMechanism::handle-job-cancel:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_jobid: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancel">JobCancel()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_job_cancel() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   *
   * Deprecated: The D-Bus method has been deprecated.
   */
  g_signal_new ("handle-job-cancel",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_job_cancel),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_job_cancel,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT);

  /**
   * CphIfaceMechanism::handle-job-cancel-purge:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_jobid: Argument passed by remote caller.
   * @arg_purge: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancelPurge">JobCancelPurge()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_job_cancel_purge() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-job-cancel-purge",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_job_cancel_purge),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_job_cancel_purge,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT, G_TYPE_BOOLEAN);

  /**
   * CphIfaceMechanism::handle-job-restart:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_jobid: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobRestart">JobRestart()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_job_restart() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-job-restart",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_job_restart),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_job_restart,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT);

  /**
   * CphIfaceMechanism::handle-job-set-hold-until:
   * @object: A #CphIfaceMechanism.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_jobid: Argument passed by remote caller.
   * @arg_job_hold_until: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobSetHoldUntil">JobSetHoldUntil()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call cph_iface_mechanism_complete_job_set_hold_until() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-job-set-hold-until",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (CphIfaceMechanismIface, handle_job_set_hold_until),
    g_signal_accumulator_true_handled,
    NULL,
      cph_iface_mechanism_method_marshal_job_set_hold_until,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT, G_TYPE_STRING);

}

/**
 * cph_iface_mechanism_call_file_get:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_resource: Argument to pass with the method invocation.
 * @arg_filename: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FileGet">FileGet()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_file_get_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_file_get_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_file_get (
    CphIfaceMechanism *proxy,
    const gchar *arg_resource,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "FileGet",
    g_variant_new ("(ss)",
                   arg_resource,
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_file_get_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_file_get().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_file_get().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_file_get_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_file_get_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_resource: Argument to pass with the method invocation.
 * @arg_filename: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FileGet">FileGet()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_file_get() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_file_get_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_resource,
    const gchar *arg_filename,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "FileGet",
    g_variant_new ("(ss)",
                   arg_resource,
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_file_put:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_resource: Argument to pass with the method invocation.
 * @arg_filename: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FilePut">FilePut()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_file_put_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_file_put_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_file_put (
    CphIfaceMechanism *proxy,
    const gchar *arg_resource,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "FilePut",
    g_variant_new ("(ss)",
                   arg_resource,
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_file_put_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_file_put().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_file_put().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_file_put_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_file_put_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_resource: Argument to pass with the method invocation.
 * @arg_filename: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FilePut">FilePut()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_file_put() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_file_put_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_resource,
    const gchar *arg_filename,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "FilePut",
    g_variant_new ("(ss)",
                   arg_resource,
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_server_get_settings:
 * @proxy: A #CphIfaceMechanismProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerGetSettings">ServerGetSettings()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_server_get_settings_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_server_get_settings_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_server_get_settings (
    CphIfaceMechanism *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ServerGetSettings",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_server_get_settings_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @out_settings: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_server_get_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_server_get_settings().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_server_get_settings_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GVariant **out_settings,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s@a{ss})",
                 out_error,
                 out_settings);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_server_get_settings_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @out_settings: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerGetSettings">ServerGetSettings()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_server_get_settings() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_server_get_settings_sync (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GVariant **out_settings,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ServerGetSettings",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s@a{ss})",
                 out_error,
                 out_settings);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_server_set_settings:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_settings: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerSetSettings">ServerSetSettings()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_server_set_settings_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_server_set_settings_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_server_set_settings (
    CphIfaceMechanism *proxy,
    GVariant *arg_settings,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ServerSetSettings",
    g_variant_new ("(@a{ss})",
                   arg_settings),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_server_set_settings_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_server_set_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_server_set_settings().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_server_set_settings_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_server_set_settings_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_settings: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerSetSettings">ServerSetSettings()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_server_set_settings() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_server_set_settings_sync (
    CphIfaceMechanism *proxy,
    GVariant *arg_settings,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ServerSetSettings",
    g_variant_new ("(@a{ss})",
                   arg_settings),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_devices_get:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_timeout: Argument to pass with the method invocation.
 * @arg_limit: Argument to pass with the method invocation.
 * @arg_include_schemes: Argument to pass with the method invocation.
 * @arg_exclude_schemes: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.DevicesGet">DevicesGet()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_devices_get_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_devices_get_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_devices_get (
    CphIfaceMechanism *proxy,
    gint arg_timeout,
    gint arg_limit,
    const gchar *const *arg_include_schemes,
    const gchar *const *arg_exclude_schemes,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "DevicesGet",
    g_variant_new ("(ii^as^as)",
                   arg_timeout,
                   arg_limit,
                   arg_include_schemes,
                   arg_exclude_schemes),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_devices_get_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @out_devices: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_devices_get().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_devices_get().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_devices_get_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GVariant **out_devices,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s@a{ss})",
                 out_error,
                 out_devices);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_devices_get_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_timeout: Argument to pass with the method invocation.
 * @arg_limit: Argument to pass with the method invocation.
 * @arg_include_schemes: Argument to pass with the method invocation.
 * @arg_exclude_schemes: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @out_devices: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.DevicesGet">DevicesGet()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_devices_get() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_devices_get_sync (
    CphIfaceMechanism *proxy,
    gint arg_timeout,
    gint arg_limit,
    const gchar *const *arg_include_schemes,
    const gchar *const *arg_exclude_schemes,
    gchar **out_error,
    GVariant **out_devices,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "DevicesGet",
    g_variant_new ("(ii^as^as)",
                   arg_timeout,
                   arg_limit,
                   arg_include_schemes,
                   arg_exclude_schemes),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s@a{ss})",
                 out_error,
                 out_devices);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_uri: Argument to pass with the method invocation.
 * @arg_ppd: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAdd">PrinterAdd()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_add_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_add_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_add (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_uri,
    const gchar *arg_ppd,
    const gchar *arg_info,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterAdd",
    g_variant_new ("(sssss)",
                   arg_name,
                   arg_uri,
                   arg_ppd,
                   arg_info,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_add_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_add().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_add().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_uri: Argument to pass with the method invocation.
 * @arg_ppd: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAdd">PrinterAdd()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_add() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_uri,
    const gchar *arg_ppd,
    const gchar *arg_info,
    const gchar *arg_location,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterAdd",
    g_variant_new ("(sssss)",
                   arg_name,
                   arg_uri,
                   arg_ppd,
                   arg_info,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_with_ppd_file:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_uri: Argument to pass with the method invocation.
 * @arg_ppd: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddWithPpdFile">PrinterAddWithPpdFile()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_add_with_ppd_file_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_add_with_ppd_file_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_add_with_ppd_file (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_uri,
    const gchar *arg_ppd,
    const gchar *arg_info,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterAddWithPpdFile",
    g_variant_new ("(sssss)",
                   arg_name,
                   arg_uri,
                   arg_ppd,
                   arg_info,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_add_with_ppd_file_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_add_with_ppd_file().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_add_with_ppd_file().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_with_ppd_file_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_with_ppd_file_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_uri: Argument to pass with the method invocation.
 * @arg_ppd: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddWithPpdFile">PrinterAddWithPpdFile()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_add_with_ppd_file() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_with_ppd_file_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_uri,
    const gchar *arg_ppd,
    const gchar *arg_info,
    const gchar *arg_location,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterAddWithPpdFile",
    g_variant_new ("(sssss)",
                   arg_name,
                   arg_uri,
                   arg_ppd,
                   arg_info,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_device:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_device: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDevice">PrinterSetDevice()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_device_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_device_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_device (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_device,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetDevice",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_device),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_device_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_device().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_device().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_device_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_device_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_device: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDevice">PrinterSetDevice()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_device() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_device_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_device,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetDevice",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_device),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_default:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDefault">PrinterSetDefault()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_default_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_default_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_default (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetDefault",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_default_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_default().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_default().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_default_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_default_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDefault">PrinterSetDefault()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_default() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_default_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetDefault",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_enabled:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_enabled: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetEnabled">PrinterSetEnabled()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_enabled_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_enabled_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_enabled (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_enabled,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetEnabled",
    g_variant_new ("(sb)",
                   arg_name,
                   arg_enabled),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_enabled_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_enabled().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_enabled().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_enabled_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_enabled_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_enabled: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetEnabled">PrinterSetEnabled()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_enabled() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_enabled_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_enabled,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetEnabled",
    g_variant_new ("(sb)",
                   arg_name,
                   arg_enabled),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_accept_jobs:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_enabled: Argument to pass with the method invocation.
 * @arg_reason: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetAcceptJobs">PrinterSetAcceptJobs()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_accept_jobs_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_accept_jobs_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_accept_jobs (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_enabled,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetAcceptJobs",
    g_variant_new ("(sbs)",
                   arg_name,
                   arg_enabled,
                   arg_reason),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_accept_jobs_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_accept_jobs().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_accept_jobs().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_accept_jobs_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_accept_jobs_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_enabled: Argument to pass with the method invocation.
 * @arg_reason: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetAcceptJobs">PrinterSetAcceptJobs()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_accept_jobs() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_accept_jobs_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_enabled,
    const gchar *arg_reason,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetAcceptJobs",
    g_variant_new ("(sbs)",
                   arg_name,
                   arg_enabled,
                   arg_reason),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_rename:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_old_name: Argument to pass with the method invocation.
 * @arg_new_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterRename">PrinterRename()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_rename_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_rename_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_rename (
    CphIfaceMechanism *proxy,
    const gchar *arg_old_name,
    const gchar *arg_new_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterRename",
    g_variant_new ("(ss)",
                   arg_old_name,
                   arg_new_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_rename_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_rename().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_rename().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_rename_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_rename_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_old_name: Argument to pass with the method invocation.
 * @arg_new_name: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterRename">PrinterRename()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_rename() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_rename_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_old_name,
    const gchar *arg_new_name,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterRename",
    g_variant_new ("(ss)",
                   arg_old_name,
                   arg_new_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_delete:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDelete">PrinterDelete()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_delete_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_delete_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_delete (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterDelete",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_delete_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_delete().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_delete().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_delete_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_delete_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDelete">PrinterDelete()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_delete() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_delete_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterDelete",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_add_printer:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_printer: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassAddPrinter">ClassAddPrinter()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_class_add_printer_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_class_add_printer_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_class_add_printer (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_printer,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ClassAddPrinter",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_printer),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_class_add_printer_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_class_add_printer().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_class_add_printer().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_add_printer_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_add_printer_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_printer: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassAddPrinter">ClassAddPrinter()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_class_add_printer() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_add_printer_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_printer,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ClassAddPrinter",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_printer),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_delete_printer:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_printer: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDeletePrinter">ClassDeletePrinter()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_class_delete_printer_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_class_delete_printer_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_class_delete_printer (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_printer,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ClassDeletePrinter",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_printer),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_class_delete_printer_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_class_delete_printer().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_class_delete_printer().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_delete_printer_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_delete_printer_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_printer: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDeletePrinter">ClassDeletePrinter()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_class_delete_printer() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_delete_printer_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_printer,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ClassDeletePrinter",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_printer),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_delete:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDelete">ClassDelete()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_class_delete_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_class_delete_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_class_delete (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ClassDelete",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_class_delete_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_class_delete().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_class_delete().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_delete_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_class_delete_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDelete">ClassDelete()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_class_delete() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_class_delete_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ClassDelete",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_info:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetInfo">PrinterSetInfo()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_info_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_info_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_info (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_info,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetInfo",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_info),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_info_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_info().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_info().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_info_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_info_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_info: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetInfo">PrinterSetInfo()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_info() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_info_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_info,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetInfo",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_info),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_location:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetLocation">PrinterSetLocation()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_location_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_location_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_location (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetLocation",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_location_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_location().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_location().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_location_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_location_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_location: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetLocation">PrinterSetLocation()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_location() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_location_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_location,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetLocation",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_shared:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_shared: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetShared">PrinterSetShared()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_shared_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_shared_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_shared (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_shared,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetShared",
    g_variant_new ("(sb)",
                   arg_name,
                   arg_shared),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_shared_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_shared().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_shared().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_shared_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_shared_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_shared: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetShared">PrinterSetShared()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_shared() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_shared_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    gboolean arg_shared,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetShared",
    g_variant_new ("(sb)",
                   arg_name,
                   arg_shared),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_job_sheets:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_start: Argument to pass with the method invocation.
 * @arg_end: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetJobSheets">PrinterSetJobSheets()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_job_sheets_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_job_sheets_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_job_sheets (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_start,
    const gchar *arg_end,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetJobSheets",
    g_variant_new ("(sss)",
                   arg_name,
                   arg_start,
                   arg_end),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_job_sheets_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_job_sheets().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_job_sheets().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_job_sheets_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_job_sheets_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_start: Argument to pass with the method invocation.
 * @arg_end: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetJobSheets">PrinterSetJobSheets()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_job_sheets() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_job_sheets_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_start,
    const gchar *arg_end,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetJobSheets",
    g_variant_new ("(sss)",
                   arg_name,
                   arg_start,
                   arg_end),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_error_policy:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_policy: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetErrorPolicy">PrinterSetErrorPolicy()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_error_policy_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_error_policy_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_error_policy (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_policy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetErrorPolicy",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_policy),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_error_policy_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_error_policy().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_error_policy().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_error_policy_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_error_policy_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_policy: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetErrorPolicy">PrinterSetErrorPolicy()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_error_policy() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_error_policy_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_policy,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetErrorPolicy",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_policy),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_op_policy:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_policy: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetOpPolicy">PrinterSetOpPolicy()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_op_policy_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_op_policy_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_op_policy (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_policy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetOpPolicy",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_policy),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_op_policy_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_op_policy().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_op_policy().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_op_policy_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_op_policy_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_policy: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetOpPolicy">PrinterSetOpPolicy()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_op_policy() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_op_policy_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_policy,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetOpPolicy",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_policy),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_users_allowed:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_users: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersAllowed">PrinterSetUsersAllowed()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_users_allowed_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_users_allowed_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_users_allowed (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *const *arg_users,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetUsersAllowed",
    g_variant_new ("(s^as)",
                   arg_name,
                   arg_users),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_users_allowed_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_users_allowed().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_users_allowed().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_users_allowed_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_users_allowed_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_users: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersAllowed">PrinterSetUsersAllowed()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_users_allowed() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_users_allowed_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *const *arg_users,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetUsersAllowed",
    g_variant_new ("(s^as)",
                   arg_name,
                   arg_users),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_users_denied:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_users: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersDenied">PrinterSetUsersDenied()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_set_users_denied_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_set_users_denied_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_set_users_denied (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *const *arg_users,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterSetUsersDenied",
    g_variant_new ("(s^as)",
                   arg_name,
                   arg_users),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_set_users_denied_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_set_users_denied().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_set_users_denied().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_users_denied_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_set_users_denied_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_users: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersDenied">PrinterSetUsersDenied()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_set_users_denied() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_set_users_denied_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *const *arg_users,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterSetUsersDenied",
    g_variant_new ("(s^as)",
                   arg_name,
                   arg_users),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_option_default:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @arg_values: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOptionDefault">PrinterAddOptionDefault()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_add_option_default_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_add_option_default_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_add_option_default (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    const gchar *const *arg_values,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterAddOptionDefault",
    g_variant_new ("(ss^as)",
                   arg_name,
                   arg_option,
                   arg_values),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_add_option_default_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_add_option_default().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_add_option_default().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_option_default_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_option_default_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @arg_values: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOptionDefault">PrinterAddOptionDefault()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_add_option_default() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_option_default_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    const gchar *const *arg_values,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterAddOptionDefault",
    g_variant_new ("(ss^as)",
                   arg_name,
                   arg_option,
                   arg_values),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_delete_option_default:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDeleteOptionDefault">PrinterDeleteOptionDefault()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_delete_option_default_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_delete_option_default_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_delete_option_default (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterDeleteOptionDefault",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_option),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_delete_option_default_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_delete_option_default().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_delete_option_default().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_delete_option_default_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_delete_option_default_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDeleteOptionDefault">PrinterDeleteOptionDefault()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_delete_option_default() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_delete_option_default_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterDeleteOptionDefault",
    g_variant_new ("(ss)",
                   arg_name,
                   arg_option),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_option:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @arg_values: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOption">PrinterAddOption()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_printer_add_option_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_printer_add_option_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_printer_add_option (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    const gchar *const *arg_values,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "PrinterAddOption",
    g_variant_new ("(ss^as)",
                   arg_name,
                   arg_option,
                   arg_values),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_printer_add_option_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_printer_add_option().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_printer_add_option().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_option_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_printer_add_option_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_option: Argument to pass with the method invocation.
 * @arg_values: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOption">PrinterAddOption()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_printer_add_option() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_printer_add_option_sync (
    CphIfaceMechanism *proxy,
    const gchar *arg_name,
    const gchar *arg_option,
    const gchar *const *arg_values,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "PrinterAddOption",
    g_variant_new ("(ss^as)",
                   arg_name,
                   arg_option,
                   arg_values),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_cancel:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancel">JobCancel()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_job_cancel_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_job_cancel_sync() for the synchronous, blocking version of this method.
 *
 * Deprecated: The D-Bus method has been deprecated.
 */
void
cph_iface_mechanism_call_job_cancel (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "JobCancel",
    g_variant_new ("(i)",
                   arg_jobid),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_job_cancel_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_job_cancel().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_job_cancel().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 *
 * Deprecated: The D-Bus method has been deprecated.
 */
gboolean
cph_iface_mechanism_call_job_cancel_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_cancel_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancel">JobCancel()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_job_cancel() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 *
 * Deprecated: The D-Bus method has been deprecated.
 */
gboolean
cph_iface_mechanism_call_job_cancel_sync (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "JobCancel",
    g_variant_new ("(i)",
                   arg_jobid),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_cancel_purge:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @arg_purge: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancelPurge">JobCancelPurge()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_job_cancel_purge_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_job_cancel_purge_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_job_cancel_purge (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    gboolean arg_purge,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "JobCancelPurge",
    g_variant_new ("(ib)",
                   arg_jobid,
                   arg_purge),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_job_cancel_purge_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_job_cancel_purge().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_job_cancel_purge().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_cancel_purge_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_cancel_purge_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @arg_purge: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancelPurge">JobCancelPurge()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_job_cancel_purge() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_cancel_purge_sync (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    gboolean arg_purge,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "JobCancelPurge",
    g_variant_new ("(ib)",
                   arg_jobid,
                   arg_purge),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_restart:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobRestart">JobRestart()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_job_restart_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_job_restart_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_job_restart (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "JobRestart",
    g_variant_new ("(i)",
                   arg_jobid),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_job_restart_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_job_restart().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_job_restart().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_restart_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_restart_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobRestart">JobRestart()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_job_restart() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_restart_sync (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "JobRestart",
    g_variant_new ("(i)",
                   arg_jobid),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_set_hold_until:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @arg_job_hold_until: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobSetHoldUntil">JobSetHoldUntil()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_call_job_set_hold_until_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_call_job_set_hold_until_sync() for the synchronous, blocking version of this method.
 */
void
cph_iface_mechanism_call_job_set_hold_until (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    const gchar *arg_job_hold_until,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "JobSetHoldUntil",
    g_variant_new ("(is)",
                   arg_jobid,
                   arg_job_hold_until),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * cph_iface_mechanism_call_job_set_hold_until_finish:
 * @proxy: A #CphIfaceMechanismProxy.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_call_job_set_hold_until().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with cph_iface_mechanism_call_job_set_hold_until().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_set_hold_until_finish (
    CphIfaceMechanism *proxy,
    gchar **out_error,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_call_job_set_hold_until_sync:
 * @proxy: A #CphIfaceMechanismProxy.
 * @arg_jobid: Argument to pass with the method invocation.
 * @arg_job_hold_until: Argument to pass with the method invocation.
 * @out_error: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobSetHoldUntil">JobSetHoldUntil()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_call_job_set_hold_until() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
cph_iface_mechanism_call_job_set_hold_until_sync (
    CphIfaceMechanism *proxy,
    gint arg_jobid,
    const gchar *arg_job_hold_until,
    gchar **out_error,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "JobSetHoldUntil",
    g_variant_new ("(is)",
                   arg_jobid,
                   arg_job_hold_until),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_error);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * cph_iface_mechanism_complete_file_get:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FileGet">FileGet()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_file_get (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_file_put:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.FilePut">FilePut()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_file_put (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_server_get_settings:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 * @settings: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerGetSettings">ServerGetSettings()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_server_get_settings (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error,
    GVariant *settings)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s@a{ss})",
                   error,
                   settings));
}

/**
 * cph_iface_mechanism_complete_server_set_settings:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ServerSetSettings">ServerSetSettings()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_server_set_settings (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_devices_get:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 * @devices: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.DevicesGet">DevicesGet()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_devices_get (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error,
    GVariant *devices)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s@a{ss})",
                   error,
                   devices));
}

/**
 * cph_iface_mechanism_complete_printer_add:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAdd">PrinterAdd()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_add (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_add_with_ppd_file:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddWithPpdFile">PrinterAddWithPpdFile()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_add_with_ppd_file (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_device:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDevice">PrinterSetDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_device (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_default:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetDefault">PrinterSetDefault()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_default (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_enabled:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetEnabled">PrinterSetEnabled()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_enabled (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_accept_jobs:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetAcceptJobs">PrinterSetAcceptJobs()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_accept_jobs (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_rename:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterRename">PrinterRename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_rename (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_delete:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDelete">PrinterDelete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_delete (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_class_add_printer:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassAddPrinter">ClassAddPrinter()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_class_add_printer (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_class_delete_printer:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDeletePrinter">ClassDeletePrinter()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_class_delete_printer (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_class_delete:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.ClassDelete">ClassDelete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_class_delete (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_info:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetInfo">PrinterSetInfo()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_info (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_location:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetLocation">PrinterSetLocation()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_location (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_shared:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetShared">PrinterSetShared()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_shared (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_job_sheets:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetJobSheets">PrinterSetJobSheets()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_job_sheets (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_error_policy:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetErrorPolicy">PrinterSetErrorPolicy()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_error_policy (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_op_policy:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetOpPolicy">PrinterSetOpPolicy()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_op_policy (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_users_allowed:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersAllowed">PrinterSetUsersAllowed()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_users_allowed (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_set_users_denied:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterSetUsersDenied">PrinterSetUsersDenied()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_set_users_denied (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_add_option_default:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOptionDefault">PrinterAddOptionDefault()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_add_option_default (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_delete_option_default:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterDeleteOptionDefault">PrinterDeleteOptionDefault()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_delete_option_default (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_printer_add_option:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.PrinterAddOption">PrinterAddOption()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_printer_add_option (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_job_cancel:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancel">JobCancel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 *
 * Deprecated: The D-Bus method has been deprecated.
 */
void
cph_iface_mechanism_complete_job_cancel (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_job_cancel_purge:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobCancelPurge">JobCancelPurge()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_job_cancel_purge (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_job_restart:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobRestart">JobRestart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_job_restart (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/**
 * cph_iface_mechanism_complete_job_set_hold_until:
 * @object: A #CphIfaceMechanism.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @error: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-opensuse-CupsPkHelper-Mechanism.JobSetHoldUntil">JobSetHoldUntil()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
cph_iface_mechanism_complete_job_set_hold_until (
    CphIfaceMechanism *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *error)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   error));
}

/* ------------------------------------------------------------------------ */

/**
 * CphIfaceMechanismProxy:
 *
 * The #CphIfaceMechanismProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * CphIfaceMechanismProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #CphIfaceMechanismProxy.
 */

struct _CphIfaceMechanismProxyPrivate
{
  GData *qdata;
};

static void cph_iface_mechanism_proxy_iface_init (CphIfaceMechanismIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (CphIfaceMechanismProxy, cph_iface_mechanism_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (CphIfaceMechanismProxy)
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_MECHANISM, cph_iface_mechanism_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (CphIfaceMechanismProxy, cph_iface_mechanism_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_MECHANISM, cph_iface_mechanism_proxy_iface_init))

#endif
static void
cph_iface_mechanism_proxy_finalize (GObject *object)
{
  CphIfaceMechanismProxy *proxy = CPH_IFACE_MECHANISM_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (cph_iface_mechanism_proxy_parent_class)->finalize (object);
}

static void
cph_iface_mechanism_proxy_get_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  GValue       *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
cph_iface_mechanism_proxy_set_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
cph_iface_mechanism_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], CPH_IFACE_TYPE_MECHANISM);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, CPH_IFACE_TYPE_MECHANISM);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
cph_iface_mechanism_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  CphIfaceMechanismProxy *proxy = CPH_IFACE_MECHANISM_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static void
cph_iface_mechanism_proxy_init (CphIfaceMechanismProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = cph_iface_mechanism_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, CPH_IFACE_TYPE_MECHANISM_PROXY, CphIfaceMechanismProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), cph_iface_mechanism_interface_info ());
}

static void
cph_iface_mechanism_proxy_class_init (CphIfaceMechanismProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = cph_iface_mechanism_proxy_finalize;
  gobject_class->get_property = cph_iface_mechanism_proxy_get_property;
  gobject_class->set_property = cph_iface_mechanism_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = cph_iface_mechanism_proxy_g_signal;
  proxy_class->g_properties_changed = cph_iface_mechanism_proxy_g_properties_changed;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (CphIfaceMechanismProxyPrivate));
#endif
}

static void
cph_iface_mechanism_proxy_iface_init (CphIfaceMechanismIface *iface G_GNUC_UNUSED)
{
}

/**
 * cph_iface_mechanism_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_proxy_new_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
cph_iface_mechanism_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (CPH_IFACE_TYPE_MECHANISM_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.opensuse.CupsPkHelper.Mechanism", NULL);
}

/**
 * cph_iface_mechanism_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with cph_iface_mechanism_proxy_new().
 *
 * Returns: (transfer full) (type CphIfaceMechanismProxy): The constructed proxy object or %NULL if @error is set.
 */
CphIfaceMechanism *
cph_iface_mechanism_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return CPH_IFACE_MECHANISM (ret);
  else
    return NULL;
}

/**
 * cph_iface_mechanism_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type CphIfaceMechanismProxy): The constructed proxy object or %NULL if @error is set.
 */
CphIfaceMechanism *
cph_iface_mechanism_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (CPH_IFACE_TYPE_MECHANISM_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.opensuse.CupsPkHelper.Mechanism", NULL);
  if (ret != NULL)
    return CPH_IFACE_MECHANISM (ret);
  else
    return NULL;
}


/**
 * cph_iface_mechanism_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like cph_iface_mechanism_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_mechanism_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See cph_iface_mechanism_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
cph_iface_mechanism_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (CPH_IFACE_TYPE_MECHANISM_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.opensuse.CupsPkHelper.Mechanism", NULL);
}

/**
 * cph_iface_mechanism_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_mechanism_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with cph_iface_mechanism_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type CphIfaceMechanismProxy): The constructed proxy object or %NULL if @error is set.
 */
CphIfaceMechanism *
cph_iface_mechanism_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return CPH_IFACE_MECHANISM (ret);
  else
    return NULL;
}

/**
 * cph_iface_mechanism_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like cph_iface_mechanism_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See cph_iface_mechanism_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type CphIfaceMechanismProxy): The constructed proxy object or %NULL if @error is set.
 */
CphIfaceMechanism *
cph_iface_mechanism_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (CPH_IFACE_TYPE_MECHANISM_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.opensuse.CupsPkHelper.Mechanism", NULL);
  if (ret != NULL)
    return CPH_IFACE_MECHANISM (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * CphIfaceMechanismSkeleton:
 *
 * The #CphIfaceMechanismSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * CphIfaceMechanismSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #CphIfaceMechanismSkeleton.
 */

struct _CphIfaceMechanismSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_cph_iface_mechanism_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  CphIfaceMechanismSkeleton *skeleton = CPH_IFACE_MECHANISM_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], CPH_IFACE_TYPE_MECHANISM);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, CPH_IFACE_TYPE_MECHANISM);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_cph_iface_mechanism_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  CphIfaceMechanismSkeleton *skeleton = CPH_IFACE_MECHANISM_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_cph_iface_mechanism_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  CphIfaceMechanismSkeleton *skeleton = CPH_IFACE_MECHANISM_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_cph_iface_mechanism_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _cph_iface_mechanism_skeleton_vtable =
{
  _cph_iface_mechanism_skeleton_handle_method_call,
  _cph_iface_mechanism_skeleton_handle_get_property,
  _cph_iface_mechanism_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
cph_iface_mechanism_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return cph_iface_mechanism_interface_info ();
}

static GDBusInterfaceVTable *
cph_iface_mechanism_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_cph_iface_mechanism_skeleton_vtable;
}

static GVariant *
cph_iface_mechanism_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  CphIfaceMechanismSkeleton *skeleton = CPH_IFACE_MECHANISM_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  if (_cph_iface_mechanism_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _cph_iface_mechanism_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _cph_iface_mechanism_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _cph_iface_mechanism_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.opensuse.CupsPkHelper.Mechanism", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static void
cph_iface_mechanism_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton G_GNUC_UNUSED)
{
}

static void cph_iface_mechanism_skeleton_iface_init (CphIfaceMechanismIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (CphIfaceMechanismSkeleton, cph_iface_mechanism_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (CphIfaceMechanismSkeleton)
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_MECHANISM, cph_iface_mechanism_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (CphIfaceMechanismSkeleton, cph_iface_mechanism_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_MECHANISM, cph_iface_mechanism_skeleton_iface_init))

#endif
static void
cph_iface_mechanism_skeleton_finalize (GObject *object)
{
  CphIfaceMechanismSkeleton *skeleton = CPH_IFACE_MECHANISM_SKELETON (object);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (cph_iface_mechanism_skeleton_parent_class)->finalize (object);
}

static void
cph_iface_mechanism_skeleton_init (CphIfaceMechanismSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = cph_iface_mechanism_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, CPH_IFACE_TYPE_MECHANISM_SKELETON, CphIfaceMechanismSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
}

static void
cph_iface_mechanism_skeleton_class_init (CphIfaceMechanismSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = cph_iface_mechanism_skeleton_finalize;

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = cph_iface_mechanism_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = cph_iface_mechanism_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = cph_iface_mechanism_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = cph_iface_mechanism_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (CphIfaceMechanismSkeletonPrivate));
#endif
}

static void
cph_iface_mechanism_skeleton_iface_init (CphIfaceMechanismIface *iface G_GNUC_UNUSED)
{
}

/**
 * cph_iface_mechanism_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>.
 *
 * Returns: (transfer full) (type CphIfaceMechanismSkeleton): The skeleton object.
 */
CphIfaceMechanism *
cph_iface_mechanism_skeleton_new (void)
{
  return CPH_IFACE_MECHANISM (g_object_new (CPH_IFACE_TYPE_MECHANISM_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for Object, ObjectProxy and ObjectSkeleton
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:CphIfaceObject
 * @title: CphIfaceObject
 * @short_description: Specialized GDBusObject types
 *
 * This section contains the #CphIfaceObject, #CphIfaceObjectProxy, and #CphIfaceObjectSkeleton types which make it easier to work with objects implementing generated types for D-Bus interfaces.
 */

/**
 * CphIfaceObject:
 *
 * The #CphIfaceObject type is a specialized container of interfaces.
 */

/**
 * CphIfaceObjectIface:
 * @parent_iface: The parent interface.
 *
 * Virtual table for the #CphIfaceObject interface.
 */

typedef CphIfaceObjectIface CphIfaceObjectInterface;
G_DEFINE_INTERFACE_WITH_CODE (CphIfaceObject, cph_iface_object, G_TYPE_OBJECT, g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_DBUS_OBJECT);)

static void
cph_iface_object_default_init (CphIfaceObjectIface *iface)
{
  /**
   * CphIfaceObject:mechanism:
   *
   * The #CphIfaceMechanism instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link>, if any.
   *
   * Connect to the #GObject::notify signal to get informed of property changes.
   */
  g_object_interface_install_property (iface, g_param_spec_object ("mechanism", "mechanism", "mechanism", CPH_IFACE_TYPE_MECHANISM, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

}

/**
 * cph_iface_object_get_mechanism:
 * @object: A #CphIfaceObject.
 *
 * Gets the #CphIfaceMechanism instance for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link> on @object, if any.
 *
 * Returns: (transfer full) (nullable): A #CphIfaceMechanism that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
 */
CphIfaceMechanism *cph_iface_object_get_mechanism (CphIfaceObject *object)
{
  GDBusInterface *ret;
  ret = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.opensuse.CupsPkHelper.Mechanism");
  if (ret == NULL)
    return NULL;
  return CPH_IFACE_MECHANISM (ret);
}


/**
 * cph_iface_object_peek_mechanism: (skip)
 * @object: A #CphIfaceObject.
 *
 * Like cph_iface_object_get_mechanism() but doesn't increase the reference count on the returned object.
 *
 * It is not safe to use the returned object if you are on another thread than the one where the #GDBusObjectManagerClient or #GDBusObjectManagerServer for @object is running.
 *
 * Returns: (transfer none) (nullable): A #CphIfaceMechanism or %NULL if @object does not implement the interface. Do not free the returned object, it is owned by @object.
 */
CphIfaceMechanism *cph_iface_object_peek_mechanism (CphIfaceObject *object)
{
  GDBusInterface *ret;
  ret = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.opensuse.CupsPkHelper.Mechanism");
  if (ret == NULL)
    return NULL;
  g_object_unref (ret);
  return CPH_IFACE_MECHANISM (ret);
}


static void
cph_iface_object_notify (GDBusObject *object, GDBusInterface *interface)
{
  _ExtendedGDBusInterfaceInfo *info = (_ExtendedGDBusInterfaceInfo *) g_dbus_interface_get_info (interface);
  /* info can be NULL if the other end is using a D-Bus interface we don't know
   * anything about, for example old generated code in this process talking to
   * newer generated code in the other process. */
  if (info != NULL)
    g_object_notify (G_OBJECT (object), info->hyphen_name);
}

/**
 * CphIfaceObjectProxy:
 *
 * The #CphIfaceObjectProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * CphIfaceObjectProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #CphIfaceObjectProxy.
 */

static void
cph_iface_object_proxy__cph_iface_object_iface_init (CphIfaceObjectIface *iface G_GNUC_UNUSED)
{
}

static void
cph_iface_object_proxy__g_dbus_object_iface_init (GDBusObjectIface *iface)
{
  iface->interface_added = cph_iface_object_notify;
  iface->interface_removed = cph_iface_object_notify;
}


G_DEFINE_TYPE_WITH_CODE (CphIfaceObjectProxy, cph_iface_object_proxy, G_TYPE_DBUS_OBJECT_PROXY,
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_OBJECT, cph_iface_object_proxy__cph_iface_object_iface_init)
                         G_IMPLEMENT_INTERFACE (G_TYPE_DBUS_OBJECT, cph_iface_object_proxy__g_dbus_object_iface_init))

static void
cph_iface_object_proxy_init (CphIfaceObjectProxy *object G_GNUC_UNUSED)
{
}

static void
cph_iface_object_proxy_set_property (GObject      *gobject,
  guint         prop_id,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec)
{
  G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
}

static void
cph_iface_object_proxy_get_property (GObject      *gobject,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec)
{
  CphIfaceObjectProxy *object = CPH_IFACE_OBJECT_PROXY (gobject);
  GDBusInterface *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.opensuse.CupsPkHelper.Mechanism");
      g_value_take_object (value, interface);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
cph_iface_object_proxy_class_init (CphIfaceObjectProxyClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->set_property = cph_iface_object_proxy_set_property;
  gobject_class->get_property = cph_iface_object_proxy_get_property;

  g_object_class_override_property (gobject_class, 1, "mechanism");
}

/**
 * cph_iface_object_proxy_new:
 * @connection: A #GDBusConnection.
 * @object_path: An object path.
 *
 * Creates a new proxy object.
 *
 * Returns: (transfer full): The proxy object.
 */
CphIfaceObjectProxy *
cph_iface_object_proxy_new (GDBusConnection *connection,
  const gchar *object_path)
{
  g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
  g_return_val_if_fail (g_variant_is_object_path (object_path), NULL);
  return CPH_IFACE_OBJECT_PROXY (g_object_new (CPH_IFACE_TYPE_OBJECT_PROXY, "g-connection", connection, "g-object-path", object_path, NULL));
}

/**
 * CphIfaceObjectSkeleton:
 *
 * The #CphIfaceObjectSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * CphIfaceObjectSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #CphIfaceObjectSkeleton.
 */

static void
cph_iface_object_skeleton__cph_iface_object_iface_init (CphIfaceObjectIface *iface G_GNUC_UNUSED)
{
}


static void
cph_iface_object_skeleton__g_dbus_object_iface_init (GDBusObjectIface *iface)
{
  iface->interface_added = cph_iface_object_notify;
  iface->interface_removed = cph_iface_object_notify;
}

G_DEFINE_TYPE_WITH_CODE (CphIfaceObjectSkeleton, cph_iface_object_skeleton, G_TYPE_DBUS_OBJECT_SKELETON,
                         G_IMPLEMENT_INTERFACE (CPH_IFACE_TYPE_OBJECT, cph_iface_object_skeleton__cph_iface_object_iface_init)
                         G_IMPLEMENT_INTERFACE (G_TYPE_DBUS_OBJECT, cph_iface_object_skeleton__g_dbus_object_iface_init))

static void
cph_iface_object_skeleton_init (CphIfaceObjectSkeleton *object G_GNUC_UNUSED)
{
}

static void
cph_iface_object_skeleton_set_property (GObject      *gobject,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  CphIfaceObjectSkeleton *object = CPH_IFACE_OBJECT_SKELETON (gobject);
  GDBusInterfaceSkeleton *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_value_get_object (value);
      if (interface != NULL)
        {
          g_warn_if_fail (CPH_IFACE_IS_MECHANISM (interface));
          g_dbus_object_skeleton_add_interface (G_DBUS_OBJECT_SKELETON (object), interface);
        }
      else
        {
          g_dbus_object_skeleton_remove_interface_by_name (G_DBUS_OBJECT_SKELETON (object), "org.opensuse.CupsPkHelper.Mechanism");
        }
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
cph_iface_object_skeleton_get_property (GObject      *gobject,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec)
{
  CphIfaceObjectSkeleton *object = CPH_IFACE_OBJECT_SKELETON (gobject);
  GDBusInterface *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.opensuse.CupsPkHelper.Mechanism");
      g_value_take_object (value, interface);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
cph_iface_object_skeleton_class_init (CphIfaceObjectSkeletonClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->set_property = cph_iface_object_skeleton_set_property;
  gobject_class->get_property = cph_iface_object_skeleton_get_property;

  g_object_class_override_property (gobject_class, 1, "mechanism");
}

/**
 * cph_iface_object_skeleton_new:
 * @object_path: An object path.
 *
 * Creates a new skeleton object.
 *
 * Returns: (transfer full): The skeleton object.
 */
CphIfaceObjectSkeleton *
cph_iface_object_skeleton_new (const gchar *object_path)
{
  g_return_val_if_fail (g_variant_is_object_path (object_path), NULL);
  return CPH_IFACE_OBJECT_SKELETON (g_object_new (CPH_IFACE_TYPE_OBJECT_SKELETON, "g-object-path", object_path, NULL));
}

/**
 * cph_iface_object_skeleton_set_mechanism:
 * @object: A #CphIfaceObjectSkeleton.
 * @interface_: (nullable): A #CphIfaceMechanism or %NULL to clear the interface.
 *
 * Sets the #CphIfaceMechanism instance for the D-Bus interface <link linkend="gdbus-interface-org-opensuse-CupsPkHelper-Mechanism.top_of_page">org.opensuse.CupsPkHelper.Mechanism</link> on @object.
 */
void cph_iface_object_skeleton_set_mechanism (CphIfaceObjectSkeleton *object, CphIfaceMechanism *interface_)
{
  g_object_set (G_OBJECT (object), "mechanism", interface_, NULL);
}


/* ------------------------------------------------------------------------
 * Code for ObjectManager client
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:CphIfaceObjectManagerClient
 * @title: CphIfaceObjectManagerClient
 * @short_description: Generated GDBusObjectManagerClient type
 *
 * This section contains a #GDBusObjectManagerClient that uses cph_iface_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc.
 */

/**
 * CphIfaceObjectManagerClient:
 *
 * The #CphIfaceObjectManagerClient structure contains only private data and should only be accessed using the provided API.
 */

/**
 * CphIfaceObjectManagerClientClass:
 * @parent_class: The parent class.
 *
 * Class structure for #CphIfaceObjectManagerClient.
 */

G_DEFINE_TYPE (CphIfaceObjectManagerClient, cph_iface_object_manager_client, G_TYPE_DBUS_OBJECT_MANAGER_CLIENT)

static void
cph_iface_object_manager_client_init (CphIfaceObjectManagerClient *manager G_GNUC_UNUSED)
{
}

static void
cph_iface_object_manager_client_class_init (CphIfaceObjectManagerClientClass *klass G_GNUC_UNUSED)
{
}

/**
 * cph_iface_object_manager_client_get_proxy_type:
 * @manager: A #GDBusObjectManagerClient.
 * @object_path: The object path of the remote object (unused).
 * @interface_name: (nullable): Interface name of the remote object or %NULL to get the object proxy #GType.
 * @user_data: User data (unused).
 *
 * A #GDBusProxyTypeFunc that maps @interface_name to the generated #GDBusObjectProxy derived and #GDBusProxy derived types.
 *
 * Returns: A #GDBusProxy derived #GType if @interface_name is not %NULL, otherwise the #GType for #CphIfaceObjectProxy.
 */
GType
cph_iface_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager G_GNUC_UNUSED, const gchar *object_path G_GNUC_UNUSED, const gchar *interface_name, gpointer user_data G_GNUC_UNUSED)
{
  static gsize once_init_value = 0;
  static GHashTable *lookup_hash;
  GType ret;

  if (interface_name == NULL)
    return CPH_IFACE_TYPE_OBJECT_PROXY;
  if (g_once_init_enter (&once_init_value))
    {
      lookup_hash = g_hash_table_new (g_str_hash, g_str_equal);
      g_hash_table_insert (lookup_hash, (gpointer) "org.opensuse.CupsPkHelper.Mechanism", (gpointer) (guintptr) (CPH_IFACE_TYPE_MECHANISM_PROXY));
      g_once_init_leave (&once_init_value, 1);
    }
  ret = (GType) (guintptr) (g_hash_table_lookup (lookup_hash, interface_name));
  if (ret == (GType) 0)
    ret = G_TYPE_DBUS_PROXY;
  return ret;
}

/**
 * cph_iface_object_manager_client_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates #GDBusObjectManagerClient using cph_iface_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc. See g_dbus_object_manager_client_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_object_manager_client_new_finish() to get the result of the operation.
 *
 * See cph_iface_object_manager_client_new_sync() for the synchronous, blocking version of this constructor.
 */
void
cph_iface_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data)
{
  g_async_initable_new_async (CPH_IFACE_TYPE_OBJECT_MANAGER_CLIENT, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "flags", flags, "name", name, "connection", connection, "object-path", object_path, "get-proxy-type-func", cph_iface_object_manager_client_get_proxy_type, NULL);
}

/**
 * cph_iface_object_manager_client_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_object_manager_client_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with cph_iface_object_manager_client_new().
 *
 * Returns: (transfer full) (type CphIfaceObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
cph_iface_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}

/**
 * cph_iface_object_manager_client_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates #GDBusObjectManagerClient using cph_iface_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc. See g_dbus_object_manager_client_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See cph_iface_object_manager_client_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type CphIfaceObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
cph_iface_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error)
{
  GInitable *ret;
  ret = g_initable_new (CPH_IFACE_TYPE_OBJECT_MANAGER_CLIENT, cancellable, error, "flags", flags, "name", name, "connection", connection, "object-path", object_path, "get-proxy-type-func", cph_iface_object_manager_client_get_proxy_type, NULL);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}


/**
 * cph_iface_object_manager_client_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like cph_iface_object_manager_client_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call cph_iface_object_manager_client_new_for_bus_finish() to get the result of the operation.
 *
 * See cph_iface_object_manager_client_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
cph_iface_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data)
{
  g_async_initable_new_async (CPH_IFACE_TYPE_OBJECT_MANAGER_CLIENT, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "flags", flags, "name", name, "bus-type", bus_type, "object-path", object_path, "get-proxy-type-func", cph_iface_object_manager_client_get_proxy_type, NULL);
}

/**
 * cph_iface_object_manager_client_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to cph_iface_object_manager_client_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with cph_iface_object_manager_client_new_for_bus().
 *
 * Returns: (transfer full) (type CphIfaceObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
cph_iface_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}

/**
 * cph_iface_object_manager_client_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like cph_iface_object_manager_client_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See cph_iface_object_manager_client_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type CphIfaceObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
cph_iface_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error)
{
  GInitable *ret;
  ret = g_initable_new (CPH_IFACE_TYPE_OBJECT_MANAGER_CLIENT, cancellable, error, "flags", flags, "name", name, "bus-type", bus_type, "object-path", object_path, "get-proxy-type-func", cph_iface_object_manager_client_get_proxy_type, NULL);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}


